//robotmain.c
//This is the main function to run the robot.
//
//The robot moves on a 7 by 7 grid and is moved forward by pressing
//ctrl-C.
//
//The robot turns east by pressing Ctrl-Z and it turns west by pressing
//Ctrl-/.
//
//It is possible to ignore these commands by opening another terminal and
//typing kill <pid>, where <pid> is the process id of this program.  Repeating
//this command will acknowledge robot movement once again.
//
//It is possible end this progra by opening another terminal and
//typing kill -9 <pid>, where <pid> is the process id of this program.

#include <string.h>
#include "robot.h"
#include <stdio.h>
#include <unistd.h>
#include <signal.h>
#include <stdlib.h>

//function prototypes

void catch_c(int);
void catch_z(int);
void catch_bksl(int);
void catch_term(int);
void set3();
void ignore3();

//global variables

int row;
int col;
int dir;

char ** slist;
int slist_size;

#define SLIST_SIZE 30000

int main(int argc, char ** argv)
{
  slist = (char **) malloc(SLIST_SIZE*sizeof(char *));
  slist_size = 0;
  initialize();
  set3();
  if(sigset(SIGTERM, catch_term) == SIG_ERR)
  {
    perror("sigset FAILED FOR SIGTERM");
    exit(1);
  }

//pause program to wait for signals

  while(1)
    pause();
}

//catch_c:  the signal handler for SIGINT
//	    moves the robot forward
//The integer signo is not used but is required to match the
//declaraion of sigset.
//
//	PARAMETERS	signo	the number of the signal caught

void catch_c(int signo)
{
  printf("SIGINT caught.\n");
  slist[slist_size] = (char *) malloc(7*sizeof(char));
  sprintf(slist[slist_size], "%s", "SIGINT");
  slist_size++;
  move(slist, slist_size);
}

//catch_z:  the signal handler for SIGTSTP
//	    makes the robot turn right
//The integer signo is not used but is required to match the
//declaraion of sigset.
//
//      PARAMETERS      signo   the number of the signal caught

void catch_z(int signo)
{
  printf("SIGTSTP caught.\n");
  slist[slist_size] = (char *) malloc(8*sizeof(char));
  sprintf(slist[slist_size], "%s", "SIGTSTP");
  slist_size++;
  turn_right();
}

//catch_bksl:  the signal handler for SIGQUIT
//	       makes the robot turn left
//The integer signo is not used but is required to match the
//declaraion of sigset.
//
//      PARAMETERS      signo   the number of the signal caught

void catch_bksl(int signo){
  printf("SIGQUIT caught.\n");
  slist[slist_size] = (char *) malloc(8*sizeof(char));
  sprintf(slist[slist_size], "%s", "SIGQUIT");
  slist_size++;
  turn_right();
}

//catch_term:	the signal handler for SIGTERM
//		toggles the three other signals on and off
//		using the slist to determine how many SIGTERMs
//		have been caught
//The integer signo is not used but is required to match the
//declaraion of sigset.
//
//      PARAMETERS      signo   the number of the signal caught

void catch_term(int signo)
{
  char * sterm = "SIGTERM";
  int count = 0;
  static int on = 0;

  printf("SIGTERM caught\n");

//First check to see if the list is empty or if the last entry to
//the list was NOT SIGTERM.  If so then ignore the 3 signals

  if((slist_size == 0) || strcmp(slist[slist_size-1],sterm) != 0)
  {
    printf("SIGINT, SIGQUIT, and SIGTERM are now being ignored.\n");
    on = 1;
    ignore3();

//If the back of the list was SIGTERM check if the size of the list is 1.
//If it is then acknowledge the 3 signals
  }
  else
  {
    if(slist_size == 1)
    {
      printf("SIGINT, SIGQUIT, and SIGTERM are now being acknowleged.\n");
      on = 0;
      set3();

//Otherwise count the number of SIGTERMs on the end of the list.
//if the number is even ignore the 3 signals
//if the number is odd recognize the 3 signals

    }
    else
    {
      while((slist_size > 0)&&(strcmp(slist[slist_size-1],sterm)!=0))
      {
        free((void *) slist[slist_size-1]);
        slist_size--;
        count++;
      }
      if(on == 0){
        printf("SIGINT, SIGQUIT, and SIGTERM are now being ignored.\n");
        on = 1;
        ignore3();
      }
      else
      {
        printf("SIGINT, SIGQUIT, and SIGTERM are now being acknowleged.\n");
        on = 0;
        set3();
      }
      for(count; count > 0; count--)
      {
        slist[slist_size] = (char *) malloc(sizeof(char)*8);
        strcpy(slist[slist_size], sterm);
        slist_size++;
      }
    }
  }
  slist[slist_size] = (char *) malloc(sizeof(char)*8);
  strcpy(slist[slist_size], sterm);
  slist_size++;
}

//ignore3:  ignores SIGINT, SIGTSTP, and SIGQUIT

void ignore3()
{
  if(sigignore(SIGINT) == -1)
  {
    perror("sigignore FAILED FOR SIGINT");
    exit(1);
  }
  if(sigignore(SIGTSTP) == -1)
  {
    perror("sigignore FAILED FOR SIGTSTP");
    exit(1);
  }
  if(sigignore(SIGQUIT) == -1)
  {
    perror("sigignore FAILED FOR SIGQUIT");
    exit(1);
  }
}

//set3:  recognizes SIGINT, SIGTSTP, and SIGQUIT

void set3()
{
  if(sigset(SIGINT, catch_c) == SIG_ERR)
  {
    perror("sigset FAILED FOR SIGINT");
    exit(1);
  }
  if(sigset(SIGTSTP, catch_z) == SIG_ERR)
  {
    perror("sigset FAILED FOR SIGTSTP");
    exit(1);
  }
  if(sigset(SIGQUIT, catch_bksl) == SIG_ERR)
  {
    perror("sigset FAILED FOR SIGQUIT");
    exit(1);
  }
}
